// $Id: TestConjunction.java,v 1.5 2004/02/27 16:30:00 m31 Exp $
//
// This file is part of the program suite "Principia Mathematica II"
// which is a working prototype for the main project:
// "Hilbert II" - http://www.qedeq.org
//
// Copyright 2001-2004  Michael Meyling <mime@qedeq.org>.
//
// "Principia Mathematica II" is free software; you can redistribute
// it and/or modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.

package com.meyling.principia.logic.basic;


import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import com.meyling.principia.argument.Argument;
import com.meyling.principia.argument.ArgumentException;
import com.meyling.principia.argument.Counter;
import com.meyling.principia.argument.Enumerator;
import com.meyling.principia.argument.PatternVariable;
import com.meyling.principia.argument.PatternVariables;


/**
 * Unit test for class Conjunction.
 *
 * @version $Revision: 1.5 $
 * @author  Michael Meyling
 */
public class TestConjunction extends TestCase {

    private Argument p0;            // A0
    private Argument p1;            // A1
    private Argument[] argument0;   // A0, A1
    private Argument[] argument1;   // A1, A1
    private Argument[] argument2;   // A1, A0
    private Argument[] argument3;   // A0, A0
    private Argument[] argument4;   // A0, A1
    private Argument[] argument5;   // A0
    private Argument[] argument6;   // A0, A1, A2
    private Argument[] argument7;   // 1
    private Argument[] argument8;   // R(0, (x0)), A1
    private Argument[] argument9;   // R(0, (x0)), A0


    /**
     * Generates new test case. The name of the testing method must
     * be given.
     * @param   name    name of testing method
     */
    public TestConjunction(String name) {
        super(name);
    }

    /**
     * Generete TestSuite .
     * @return reference to TestSuite of this class.
     */
    public static Test suite() {
        TestSuite suite = new TestSuite();
        suite.addTest(new TestConjunction("TestConstructor"));
        suite.addTest(new TestConjunction("TestEquals"));
        suite.addTest(new TestConjunction("TestMatches"));
        suite.addTest(new TestConjunction("TestReplace"));
        return suite;
    }


    /**
     * Make some initialisations for a test.
     */
    public void setUp() {

        try {
            p0 = new PropositionVariable(
                    new Argument[] {
                        new Counter(0)
                    }
                );

            p1 = new PropositionVariable(
                    new Argument[] {
                        new Counter(1)
                    }
                );

            argument0
                = new Argument[] {
                    new PropositionVariable(
                        new Argument[] {
                            new Counter(0)
                        }
                    ),
                    new PropositionVariable(
                        new Argument[] {
                            new Counter(1)
                        }
                    )
                };

            argument1
                = new Argument[] {
                    new PropositionVariable(
                        new Argument[] {
                            new Counter(1)
                        }
                    ),
                    new PropositionVariable(
                        new Argument[] {
                            new Counter(1)
                        }
                    )
                };
            argument2
                = new Argument[] {
                    new PropositionVariable(
                        new Argument[] {
                            new Counter(1)
                        }
                    ),
                    new PropositionVariable(
                        new Argument[] {
                            new Counter(0)
                        }
                    )
                };

            argument3
                = new Argument[] {
                    new PropositionVariable(
                        new Argument[] {
                            new Counter(0)
                        }
                    ),
                    new PropositionVariable(
                        new Argument[] {
                            new Counter(0)
                        }
                    )
                };

            argument4
                = new Argument[] {
                    new PropositionVariable(
                        new Argument[] {
                            new Counter(0)
                        }
                    ),
                    new PropositionVariable(
                        new Argument[] {
                            new Counter(1)
                        }
                    )
                };


            argument5
                = new Argument[] {
                    new PropositionVariable(
                        new Argument[] {
                            new Counter(0)
                        }
                    )
                };


            argument6
                = new Argument[] {
                    new PropositionVariable(
                        new Argument[] {
                            new Counter(0)
                        }
                    ),
                    new PropositionVariable(
                        new Argument[] {
                            new Counter(1)
                        }
                    ),
                    new PropositionVariable(
                        new Argument[] {
                            new Counter(2)
                        }
                    )
                };


            argument7
                = new Argument[] {
                    new Counter(1)
                };


            argument9
                = new Argument[] {
                    new PredicateVariable(
                        new Argument[] {
                            new Counter(0),
                            new RegularArgumentList(
                                new Argument[] {
                                    new SubjectVariable(
                                        new Argument[] {
                                            new Counter(0)
                                        }
                                    )
                                }
                            )
                        }
                    ),
                    new PropositionVariable(
                        new Argument[] {
                            new Counter(0)
                        }
                    )
                };


            argument8
                = new Argument[] {
                    new PredicateVariable(
                        new Argument[] {
                            new Counter(0),
                            new RegularArgumentList(
                                new Argument[] {
                                    new SubjectVariable(
                                        new Argument[] {
                                            new Counter(0)
                                        }
                                    )
                                }
                            )
                        }
                    ),
                    new PropositionVariable(
                        new Argument[] {
                            new Counter(1)
                        }
                    )
                };

        } catch (ArgumentException e) {
            e.printStackTrace();
            throw new IllegalArgumentException(e.getMessage());
        }

    }


    /**
     * Free resources that were used in a test.
     */
    public void tearDown() {
        p0 = null;
        p1 = null;
        argument0 = null;
        argument1 = null;
        argument2 = null;
        argument3 = null;
        argument4 = null;
        argument5 = null;
        argument6 = null;
        argument7 = null;
        argument8 = null;
        argument9 = null;
    }


    /**
     * Test the Constructor.
     */
    public void TestConstructor() {

        Formula formula = null;
        try {
            formula = new Conjunction(argument0);           // A0 n A1
            formula = new Conjunction(argument1);           // A1 n A1
            formula = new Conjunction(argument2);           // A1 n A0
            formula = new Conjunction(argument3);           // A0 n A0
            formula = new Conjunction(argument4);           // A0 n A1
        } catch (ArgumentException e) {
            e.printStackTrace();
            throw new IllegalArgumentException(e.getMessage());
        }
        try {
            formula = new Conjunction(null);
            fail("IllegalArgumentException expected");
        } catch (IllegalArgumentException ilae) {
        } catch (ArgumentException e) {
            e.printStackTrace();
            throw new IllegalArgumentException(e.getMessage());
        }
        try {
            formula = new Conjunction(argument5);
            fail("ArgumentException expected");
        } catch (ArgumentException ilae) {
        }
        try {
            formula = new Conjunction(argument6);
            fail("ArgumentException expected");
        } catch (ArgumentException ilae) {
        }
        try {
            formula = new Conjunction(argument7);
            fail("ArgumentException expected");
        } catch (ArgumentException ilae) {
        }

    }

    /**
     * Test the method {@link Conjunction#equals}.
     */
    public void TestEquals() {

        final Formula formula0;
        final Formula formula1;
        final Formula formula2;
        final Formula formula3;
        final Formula formula4;
        final Formula formula5;
        try {
            formula0 = new Conjunction(argument0);  // A0 n A1
            formula1 = new Conjunction(argument1);  // A1 n A1
            formula2 = new Conjunction(argument2);  // A1 n A0
            formula3 = new Conjunction(argument3);  // A0 n A0
            formula4 = new Conjunction(argument4);  // A0 n A1
            formula5 = null;
        } catch (ArgumentException e) {
            e.printStackTrace();
            throw new IllegalArgumentException(e.getMessage());
        }

        assertTrue(formula0.equals(formula0));
        assertTrue(!formula0.equals(formula1));
        assertTrue(!formula0.equals(formula2));
        assertTrue(!formula0.equals(formula3));
        assertTrue(formula0.equals(formula4));
        assertTrue(!formula1.equals(formula0));
        assertTrue(formula1.equals(formula1));
        assertTrue(!formula1.equals(formula2));
        assertTrue(!formula1.equals(formula3));
        assertTrue(!formula1.equals(formula4));
        assertTrue(!formula2.equals(formula0));
        assertTrue(!formula2.equals(formula1));
        assertTrue(formula2.equals(formula2));
        assertTrue(!formula2.equals(formula3));
        assertTrue(!formula2.equals(formula4));
        assertTrue(!formula3.equals(formula0));
        assertTrue(!formula3.equals(formula1));
        assertTrue(!formula3.equals(formula2));
        assertTrue(formula3.equals(formula3));
        assertTrue(!formula3.equals(formula4));
        assertTrue(formula4.equals(formula0));
        assertTrue(!formula4.equals(formula1));
        assertTrue(!formula4.equals(formula2));
        assertTrue(!formula4.equals(formula3));
        assertTrue(formula4.equals(formula4));

    }


    /**
     * Test the method {@link Conjunction#matches}.
     */
    public void TestMatches() {

        Formula formula0;
        Formula formula1;
        Formula formula2;
        Formula formula3;
        Formula formula4;
        Formula formula5;
        Formula formula8;
        Formula formula9;
        PatternVariables variables = null;
        PatternVariable pat0;
        PatternVariable pat1;
        PatternVariable pat4;
        PatternVariable pat00;

        PatternVariable result = null;
        try {
            formula0 = new Conjunction(argument0);  // A0 n A1
            formula1 = new Conjunction(argument1);  // A1 n A1
            formula2 = new Conjunction(argument2);  // A1 n A0
            formula3 = new Conjunction(argument3);  // A0 n A0
            formula4 = new Conjunction(argument4);  // A0 n A1
            formula5 = null;
            formula8 = new Conjunction(argument8);  // R(0, ()) n A1
            formula9 = new Conjunction(argument9);  // R(0, ()) n A0
            pat0 = new PatternVariable(
                new Argument[] {
                    new Counter(0)
                }
            );
            pat1 = new PatternVariable(
                new Argument[] {
                    new Counter(1)
                }
            );
            pat4 = new PatternVariable(
                new Argument[] {
                    new Counter(4)
                }
            );
            pat00 = new PatternVariable(
                new Argument[] {
                    new Counter(0)
                }
            );

        } catch (ArgumentException e) {
            e.printStackTrace();
            throw new IllegalArgumentException(e.getMessage());
        }


        // testing method: matches(Argument)

        // A0 n A1 matches A0 n A1
        assertTrue(formula0.matches(formula0));

        // A0 n A1 matches @0
        assertTrue(formula0.matches(pat00));

        // A0 n A1 matches A1
        assertTrue(formula0.matches(p1));

        // A0 dosn't match A0 n A1
        assertTrue(!p0.matches(formula0));

        // @0 couldn't match anything because it is a PatternVariable
        //  itself
        try {
            pat00.matches(formula0);
        } catch (IllegalArgumentException e) {
            // expected
        }

        // @0 couldn't match anything because it contains
        //  PatternVariables
        try {
            p0.matches(formula0);
        } catch (IllegalArgumentException e) {
            // expected
        }

    }


    /**
     * Test the methods {@link Conjunction#replace}.
     */
    public void TestReplace() {

        Formula formula0;
        Formula formula1;
        Formula formula3;
        Formula formula8;
        PropositionVariable p0;
        PropositionVariable p1;
        try {
            formula0 = new Conjunction(argument0);  // A0 n A1
            formula1 = new Conjunction(argument1);  // A1 n A1
            formula3 = new Conjunction(argument3);  // A0 n A0
            formula8 = new Conjunction(argument8);  // R(0, ()) n A1

            p0 = new PropositionVariable(
                new Argument[] {
                    new Counter(0)
                }
            );
            p1 = new PropositionVariable(
                new Argument[] {
                    new Counter(1)
                }
            );
        } catch (ArgumentException e) {
            e.printStackTrace();
            throw new IllegalArgumentException(e.getMessage());
        }

        try {
            // test method: replace(Argument, Argument)

            // replace A0 by A1 in      A0 n A1
            Formula result = (Formula) formula0.replace(p0, p1);
            //          .. must be      A1 n A1
            assertTrue(result.equals(formula1));

            // replace A1 by A0 in      A0 n A1
            result = (Formula) formula0.replace(p1, p0);
            //          .. must be      A0 n A0
            assertTrue(result.equals(formula3));

            // replace A1 by A0 in      A0 n A0
            result = (Formula) result.replace(p1, p0);
            //          .. must be      A0 n A0
            assertTrue(result.equals(formula3));


            Formula expected = new Conjunction(
                new Argument[] {
                    new PredicateVariable(
                        new Argument[] {
                            new Counter(9),
                            new RegularArgumentList(
                                new Argument[] {
                                    new SubjectVariable(
                                        new Argument[] {
                                            new Counter(9)
                                        }
                                    )
                                }
                            )
                        }
                    ),
                    new PropositionVariable(
                        new Argument[] {
                            new Counter(1)
                        }
                    )
                }
            );

            // replacce 0 by 1 in       R(0, (x0)) n A1
            result = (Formula) formula8.replace(new Counter(0), new Counter(9));
            //          .. must be      R(1, (x1)) n A1
            assertTrue(result.equals(expected));


            // replace(Enumerator counter, int occurance,
            //        Argument searchPattern, Argument replacePattern)


            // replace first occurence of 0 in A0 n A0 by 2
            result = (Formula)  formula3.replace(new Enumerator(), 0,
                new PropositionVariable(
                    new Argument[] {
                        new PatternVariable(
                            new Argument[] {
                                new Counter(4)
                            }
                        )
                    }
                ),
                new PropositionVariable(
                    new Argument[] {
                        new Counter(2)
                    }
                )
            );
            // must be A2 n A0
            assertTrue(result.equals(
                new Conjunction(
                    new Argument[] {
                        new PropositionVariable(
                            new Argument[] {
                                new Counter(2)
                            }
                        ),
                        new PropositionVariable(
                            new Argument[] {
                                new Counter(0)
                            }
                        )
                    }
                )
            ));

            // replace second occurence of 0 in A0 n A0 by 2
            result = (Formula)  formula3.replace(new Enumerator(), 1,
                new PropositionVariable(
                    new Argument[] {
                        new PatternVariable(
                            new Argument[] {
                                new Counter(4)
                            }
                        )
                    }
                ),
                new PropositionVariable(
                    new Argument[] {
                        new Counter(2)
                    }
                )
            );
            // must be A0 n A2
            assertTrue(result.equals(
                new Conjunction(
                    new Argument[] {
                        new PropositionVariable(
                            new Argument[] {
                                new Counter(0)
                            }
                        ),
                        new PropositionVariable(
                            new Argument[] {
                                new Counter(2)
                            }
                        )
                    }
                )
            ));


            // replace third occurence of 0 in A0 n A0 by 2
            result = (Formula)  formula3.replace(new Enumerator(), 2,
                new PropositionVariable(
                    new Argument[] {
                        new PatternVariable(
                            new Argument[] {
                                new Counter(4)
                            }
                        )
                    }
                ),
                new PropositionVariable(
                    new Argument[] {
                        new Counter(2)
                    }
                )
            );
            // must be A0 n A0 (because third occurance wasn't found)
            assertTrue(result.equals(
                new Conjunction(
                    new Argument[] {
                        new PropositionVariable(
                            new Argument[] {
                                new Counter(0)
                            }
                        ),
                        new PropositionVariable(
                            new Argument[] {
                                new Counter(0)
                            }
                        )
                    }
                )
            ));


            // replaceBoundSubjectVariable(Enumerator counter, int occurance,
            //        SubjectVariable search, SubjectVariable replacement)

            // replace first bound occurence of x2 by x7 in A(x2: R0(x2, x7))
            final Formula formula11 =
                new UniversalQuantifier(
                    new Argument[] {
                        new SubjectVariable(
                            new Argument[] {
                                new Counter(2)
                            }
                        ),
                        new PredicateVariable(
                            new Argument[] {
                                new Counter(0),
                                new RegularArgumentList(
                                    new Argument[] {
                                        new SubjectVariable(
                                            new Argument[] {
                                                new Counter(2)
                                            }
                                        ),
                                        new SubjectVariable(
                                            new Argument[] {
                                                new Counter(7)
                                            }
                                        )
                                    }
                                )
                            }
                        )
                    }
                );

            result = (Formula) formula11.replaceBoundSubjectVariable(
                new Enumerator(),
                0,
                new SubjectVariable(
                    new Argument[] {
                        new Counter(2)
                    }
                ),
                new SubjectVariable(
                    new Argument[] {
                        new Counter(7)
                    }
                )
            );
            // must be A(x7: R0(x7, x7)
            assertTrue(result.equals(
                new UniversalQuantifier(
                    new Argument[] {
                        new SubjectVariable(
                            new Argument[] {
                                new Counter(7)
                            }
                        ),
                        new PredicateVariable(
                            new Argument[] {
                                new Counter(0),
                                new RegularArgumentList(
                                    new Argument[] {
                                        new SubjectVariable(
                                            new Argument[] {
                                                new Counter(7)
                                            }
                                        ),
                                        new SubjectVariable(
                                            new Argument[] {
                                                new Counter(7)
                                            }
                                        )
                                    }
                                )
                            }
                        )
                    }
                )
            ));

            // replace first bound occurence of x2 by x7 in
            //  A(x2: R0(x2, x7)) n A(x2: R0(x2, x8))
            final Formula formula12 =
                new Conjunction(
                    new Argument[] {
                        new UniversalQuantifier(
                            new Argument[] {
                                new SubjectVariable(
                                    new Argument[] {
                                        new Counter(2)
                                    }
                                ),
                                new PredicateVariable(
                                    new Argument[] {
                                        new Counter(0),
                                        new RegularArgumentList(
                                            new Argument[] {
                                                new SubjectVariable(
                                                    new Argument[] {
                                                        new Counter(2)
                                                    }
                                                ),
                                                new SubjectVariable(
                                                    new Argument[] {
                                                        new Counter(7)
                                                    }
                                                )
                                            }
                                        )
                                    }
                                )
                            }
                        ),
                        new UniversalQuantifier(
                            new Argument[] {
                                new SubjectVariable(
                                    new Argument[] {
                                        new Counter(2)
                                    }
                                ),
                                new PredicateVariable(
                                    new Argument[] {
                                        new Counter(0),
                                        new RegularArgumentList(
                                            new Argument[] {
                                                new SubjectVariable(
                                                    new Argument[] {
                                                        new Counter(2)
                                                    }
                                                ),
                                                new SubjectVariable(
                                                    new Argument[] {
                                                        new Counter(8)
                                                    }
                                                )
                                            }
                                        )
                                    }
                                )
                            }
                        )
                    }
                );

            result = (Formula) formula12.replaceBoundSubjectVariable(
                new Enumerator(),
                0,
                new SubjectVariable(
                    new Argument[] {
                        new Counter(2)
                    }
                ),
                new SubjectVariable(
                    new Argument[] {
                        new Counter(7)
                    }
                )
            );
            //  must be A(x7: R0(x7, x7)) n A(x2: R0(x2, x8))
            assertTrue(result.equals(
                new Conjunction(
                    new Argument[] {
                        new UniversalQuantifier(
                            new Argument[] {
                                new SubjectVariable(
                                    new Argument[] {
                                        new Counter(7)
                                    }
                                ),
                                new PredicateVariable(
                                    new Argument[] {
                                        new Counter(0),
                                        new RegularArgumentList(
                                            new Argument[] {
                                                new SubjectVariable(
                                                    new Argument[] {
                                                        new Counter(7)
                                                    }
                                                ),
                                                new SubjectVariable(
                                                    new Argument[] {
                                                        new Counter(7)
                                                    }
                                                )
                                            }
                                        )
                                    }
                                )
                            }
                        ),
                        new UniversalQuantifier(
                            new Argument[] {
                                new SubjectVariable(
                                    new Argument[] {
                                        new Counter(2)
                                    }
                                ),
                                new PredicateVariable(
                                    new Argument[] {
                                        new Counter(0),
                                        new RegularArgumentList(
                                            new Argument[] {
                                                new SubjectVariable(
                                                    new Argument[] {
                                                        new Counter(2)
                                                    }
                                                ),
                                                new SubjectVariable(
                                                    new Argument[] {
                                                        new Counter(8)
                                                    }
                                                )
                                            }
                                        )
                                    }
                                )
                            }
                        )
                    }
                )
            ));

            // replace second bound occurence of x2 by x9 in
            //  A(x2: R0(x2, x7)) n A(x2: R0(x2, x8))
            final Formula formula13 =
                new Conjunction(
                    new Argument[] {
                        new UniversalQuantifier(
                            new Argument[] {
                                new SubjectVariable(
                                    new Argument[] {
                                        new Counter(2)
                                    }
                                ),
                                new PredicateVariable(
                                    new Argument[] {
                                        new Counter(0),
                                        new RegularArgumentList(
                                            new Argument[] {
                                                new SubjectVariable(
                                                    new Argument[] {
                                                        new Counter(2)
                                                    }
                                                ),
                                                new SubjectVariable(
                                                    new Argument[] {
                                                        new Counter(7)
                                                    }
                                                )
                                            }
                                        )
                                    }
                                )
                            }
                        ),
                        new UniversalQuantifier(
                            new Argument[] {
                                new SubjectVariable(
                                    new Argument[] {
                                        new Counter(2)
                                    }
                                ),
                                new PredicateVariable(
                                    new Argument[] {
                                        new Counter(0),
                                        new RegularArgumentList(
                                            new Argument[] {
                                                new SubjectVariable(
                                                    new Argument[] {
                                                        new Counter(2)
                                                    }
                                                ),
                                                new SubjectVariable(
                                                    new Argument[] {
                                                        new Counter(8)
                                                    }
                                                )
                                            }
                                        )
                                    }
                                )
                            }
                        )
                    }
                );

            result = (Formula) formula13.replaceBoundSubjectVariable(
                new Enumerator(),
                1,
                new SubjectVariable(
                    new Argument[] {
                        new Counter(2)
                    }
                ),
                new SubjectVariable(
                    new Argument[] {
                        new Counter(9)
                    }
                )
            );

            //  must be A(x2: R0(x2, x7)) n A(x9: R0(x9, x8))
            assertTrue(result.equals(
                new Conjunction(
                    new Argument[] {
                        new UniversalQuantifier(
                            new Argument[] {
                                new SubjectVariable(
                                    new Argument[] {
                                        new Counter(2)
                                    }
                                ),
                                new PredicateVariable(
                                    new Argument[] {
                                        new Counter(0),
                                        new RegularArgumentList(
                                            new Argument[] {
                                                new SubjectVariable(
                                                    new Argument[] {
                                                        new Counter(2)
                                                    }
                                                ),
                                                new SubjectVariable(
                                                    new Argument[] {
                                                        new Counter(7)
                                                    }
                                                )
                                            }
                                        )
                                    }
                                )
                            }
                        ),
                        new UniversalQuantifier(
                            new Argument[] {
                                new SubjectVariable(
                                    new Argument[] {
                                        new Counter(9)
                                    }
                                ),
                                new PredicateVariable(
                                    new Argument[] {
                                        new Counter(0),
                                        new RegularArgumentList(
                                            new Argument[] {
                                                new SubjectVariable(
                                                    new Argument[] {
                                                        new Counter(9)
                                                    }
                                                ),
                                                new SubjectVariable(
                                                    new Argument[] {
                                                        new Counter(8)
                                                    }
                                                )
                                            }
                                        )
                                    }
                                )
                            }
                        )
                    }
                )
            ));

            // replace second bound occurence of x2 by x7 in
            //  A(x2: R0(x2, x7)) n A(x2: R0(x2, x8))
            final Formula formula14 =
                new Conjunction(
                    new Argument[] {
                        new UniversalQuantifier(
                            new Argument[] {
                                new SubjectVariable(
                                    new Argument[] {
                                        new Counter(2)
                                    }
                                ),
                                new PredicateVariable(
                                    new Argument[] {
                                        new Counter(0),
                                        new RegularArgumentList(
                                            new Argument[] {
                                                new SubjectVariable(
                                                    new Argument[] {
                                                        new Counter(2)
                                                    }
                                                ),
                                                new SubjectVariable(
                                                    new Argument[] {
                                                        new Counter(7)
                                                    }
                                                )
                                            }
                                        )
                                    }
                                )
                            }
                        ),
                        new UniversalQuantifier(
                            new Argument[] {
                                new SubjectVariable(
                                    new Argument[] {
                                        new Counter(2)
                                    }
                                ),
                                new PredicateVariable(
                                    new Argument[] {
                                        new Counter(0),
                                        new RegularArgumentList(
                                            new Argument[] {
                                                new SubjectVariable(
                                                    new Argument[] {
                                                        new Counter(2)
                                                    }
                                                ),
                                                new SubjectVariable(
                                                    new Argument[] {
                                                        new Counter(8)
                                                    }
                                                )
                                            }
                                        )
                                    }
                                )
                            }
                        )
                    }
                );

            try {
                formula14.replaceBoundSubjectVariable(
                    new Enumerator(),
                    1,
                    new SubjectVariable(
                        new Argument[] {
                            new Counter(2)
                        }
                    ),
                    new SubjectVariable(
                        new Argument[] {
                            new Counter(7)
                        }
                    )
                 );
                 fail("ArgumentException expected");
            } catch (ArgumentException e) {
            }

        } catch (ArgumentException e) {
            e.printStackTrace();
            throw new IllegalArgumentException(e.getMessage());
        }
    }



}
