// $Id: ExistentialQuantifier.java,v 1.5 2004/02/27 16:30:00 m31 Exp $
//
// This file is part of the program suite "Principia Mathematica II"
// which is a working prototype for the main project:
// "Hilbert II" - http://www.qedeq.org
//
// Copyright 2001-2004  Michael Meyling <mime@qedeq.org>.
//
// "Principia Mathematica II" is free software; you can redistribute
// it and/or modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.

package com.meyling.principia.logic.basic;


import com.meyling.principia.argument.Argument;
import com.meyling.principia.argument.ArgumentException;


/**
 * Existential quantifier, logical &quot;exists&quot;.
 * @version $Revision: 1.5 $
 * @author  Michael Meyling
 */
public class ExistentialQuantifier extends Quantifier {


    /**
     * Constructs a logical existential quantifier.
     *
     * @param   arguments    subject variable and formula to operate on
     * @throws  ArgumentException    if there are not exactly two
     *          arguments ore the first argument is not an instance of
     *          {@link SubjectVariableMarker} or the second argument
     *          is not an instance of {@link Formula} or
     *          the subject variable is not free in the formula
     */
    public ExistentialQuantifier(final Argument[] arguments)
            throws ArgumentException {
        super(arguments);
    }


    /** @inherited */
    public final Argument create(final Argument[] arguments)
            throws ArgumentException {
        return new ExistentialQuantifier(arguments);
    }


    /** @inherited */
    public final String toString() {
        return "E " + getVariable().toString() + " "
            + getPartFormula(0).toString();
    }


}
