// $Id: Email.java,v 1.5 2004/02/27 16:30:00 m31 Exp $
//
// This file is part of the program suite "Principia Mathematica II"
// which is a working prototype for the main project:
// "Hilbert II" - http://www.qedeq.org
//
// Copyright 2001-2004  Michael Meyling <mime@qedeq.org>.
//
// "Principia Mathematica II" is free software; you can redistribute
// it and/or modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.

package com.meyling.principia.module;


import com.meyling.principia.argument.AbstractArgumentList;
import com.meyling.principia.argument.Argument;
import com.meyling.principia.argument.ArgumentException;
import com.meyling.principia.argument.Text;


/**
 * An email address.
 *
 * @version $Revision: 1.5 $
 * @author  Michael Meyling
 */
public class Email extends AbstractArgumentList
        implements Argument, Textable {


    /**
     * Constructs a module description.
     *
     * @param   arguments     {@link Text} to be the email address
     * @throws  ArgumentException    if there is not exactly one
     *          argument or the first argument is not an instance
     *          of {@link Text}.
     */
    public Email(final Argument[] arguments)
            throws ArgumentException {
        super(arguments);

        if (arguments.length != 1) {
            throw new ArgumentException(
                ModuleConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_NUMBER,
                ModuleCreator.getName(this.getClass())
                + ModuleConstants.OPERATOR_WITH_ONE_ARGUMENT);
        }
        if (!(arguments[0] instanceof Text)) {
            throw new ArgumentException(
                ModuleConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ModuleCreator.getName(this.getClass())
                + ModuleConstants.FIRST_ARGUMENT_TYPE
                + ModuleConstants.QUOTED_STRING,
                arguments[0]);
        }
    }


    /** @inherited */
    public final String getText() {
        return ((Text) getArgument(0)).getText();
    }


    /** @inherited */
    public final Argument create(final Argument[] arguments)
            throws ArgumentException {
        return new Email(arguments);
    }


    /** @inherited */
    public final String toString() {
        return "Email " + getArgument(0).toString();
    }


}
